package net.alviano.unsafe;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

public class Concessionaria {
	private List<Veicolo> veicoli = new ArrayList<Veicolo>();
	
	public void aggiungiVeicolo(Veicolo v) {
		this.veicoli.add(v);
	}
	
	public void rimuoviVeicolo(Veicolo v) {
		this.veicoli.remove(v);
	}
	
	void stampaVeicoli() {
		System.out.println(veicoli);
	}
	
	void ordinaVeicoliPerPrezzo() {
		this.veicoli.sort(new Comparator<>() {
			@Override
			public int compare(Veicolo arg0, Veicolo arg1) {
				return arg0.getPrezzo().compareTo(arg1.getPrezzo());
			}
		});
	}
	
	void ordinaVeicoliPerNome() {
		this.veicoli.sort(new Comparator<>() {
			@Override
			public int compare(Veicolo arg0, Veicolo arg1) {
				return arg0.getNome().compareTo(arg1.getNome());
			}
		});
	}
	
	void calcolaValore() {
		Double res = 0.0;
		for(Veicolo v : this.veicoli) {
			res += v.getPrezzo();
		}
		System.out.println(String.format("Il valore è %.2f", res));
	}
	
	void salvaSuFile(String filename) {
		List<String> list = new ArrayList<String>();
		for(Veicolo v : this.veicoli) {
			list.add(v.toSaveFormat());
		}
	    try {
			Files.write(Paths.get(filename), list);
		} catch (IOException e) {
			System.err.println("Concessionaria: Errore durante il salvataggio dei dati");
		}
	}
	
	void leggiDaFile(String filename) {
		this.veicoli.clear();
		try {
			List<String> lines = Files.readAllLines(Paths.get(filename));
			for(String line : lines) {
				String[] atts = line.split(";");
				String tipo = atts[0];
				String targa = atts[1];
				String casa = atts[2];
				String nome = atts[3];
				Double prezzo = Double.parseDouble(atts[4]);
				switch (tipo) {
				case "Veicolo": {
					this.veicoli.add(new Veicolo(targa, casa, nome, prezzo));
					break;
				}
				case "Automobile": {
					this.veicoli.add(new Automobile(targa, casa, nome, prezzo));
					break;
				}
				case "Moto": {
					this.veicoli.add(new Moto(targa, casa, nome, prezzo));
					break;
				}
				default:
					throw new IllegalStateException("Concessionaria: Valore invalido durante il caricamento dei dati");
				}
			}
		} catch (IOException e) {
			throw new IllegalStateException("Concessionaria: Errore durante il caricamento dei dati");
		}
	}
	
	public static void main(String[] args) {
		Concessionaria concessionaria = new Concessionaria();
		concessionaria.leggiDaFile("concessionaria.csv");
//		Automobile cc = new Automobile("AB123CD", "Volkswagen", "CC", 19000);
//		concessionaria.aggiungiVeicolo(cc);
		concessionaria.calcolaValore();
		concessionaria.salvaSuFile("concessionaria.csv");
	}
	
}
