package net.alviano.safe;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

public final class Concessionaria {
	private final List<Veicolo> veicoli = new ArrayList<Veicolo>();
	
	public void aggiungiVeicolo(Veicolo v) {
		this.veicoli.add(v);
	}
	
	public void rimuoviVeicolo(Veicolo v) {
		this.veicoli.remove(v);
	}
	
	public List<Veicolo> veicoli() {
		return Collections.unmodifiableList(veicoli);
	}
	
	public void ordinaVeicoliPerPrezzo() {
		this.veicoli.sort(new Comparator<>() {
			@Override
			public int compare(Veicolo arg0, Veicolo arg1) {
				return arg0.prezzo().compareTo(arg1.prezzo());
			}
		});
	}
	
	public void ordinaVeicoliPerNome() {
		this.veicoli.sort(new Comparator<>() {
			@Override
			public int compare(Veicolo arg0, Veicolo arg1) {
				return arg0.modello().compareTo(arg1.modello());
			}
		});
	}
	
	public Prezzo calcolaValore() {
		Prezzo res = new Prezzo(0);
		for(Veicolo v : this.veicoli) {
			res = res.add(v.prezzo());
		}
		return res;
	}
	
	public void salvaSuFile(String filename) throws IOException {
		List<String> list = new ArrayList<String>();
		for(Veicolo v : this.veicoli) {
			list.add(v.toSaveFormat());
		}
		Files.write(Paths.get(filename), list);
	}
	
	public void leggiDaFile(String filename) throws IOException {
		this.veicoli.clear();
		List<String> lines = Files.readAllLines(Paths.get(filename));
		for(String line : lines) {
			String[] atts = line.split(";");
			String tipo = atts[0];
			Targa targa = new Targa(atts[1]);
			CasaProduttrice casa = new CasaProduttrice(atts[2]);
			ModelloVeicolo nome = new ModelloVeicolo(atts[3]);
			Prezzo prezzo = Prezzo.parse(atts[4]);
			switch (tipo) {
			case "Automobile": {
				this.veicoli.add(new Automobile(targa, casa, nome, prezzo));
				break;
			}
			case "Moto": {
				this.veicoli.add(new Moto(targa, casa, nome, prezzo));
				break;
			}
			default:
				throw new IllegalStateException("Concessionaria: Valore invalido durante il caricamento dei dati");
			}
		}
	}
	
	public static void main(String[] args) {
		try {
			Concessionaria concessionaria = new Concessionaria();
			concessionaria.leggiDaFile("concessionaria.csv");
	//		Automobile cc = new Automobile("AB123CD", "Volkswagen", "CC", 19000);
	//		concessionaria.aggiungiVeicolo(cc);
			concessionaria.calcolaValore();
			concessionaria.salvaSuFile("concessionaria.csv");
		} catch (Exception e) {
			System.out.println("Panic error!");
		}
	}
	
}
