package net.alviano.safe;

import static org.apache.commons.lang3.Validate.*;

public abstract class Veicolo {
	private final Targa targa;
	private final CasaProduttrice casaProduttrice;
	private final ModelloVeicolo modello;
	private final Prezzo prezzo;
	
	public Veicolo(Targa targa, CasaProduttrice casaProduttrice, ModelloVeicolo modello, Prezzo prezzo) {
		this.targa = notNull(targa);
		this.casaProduttrice = notNull(casaProduttrice);
		this.modello = notNull(modello);
		this.prezzo = notNull(prezzo);
	}
	
	final public String tipo() {
		return this.getClass().getSimpleName();
	}
	
	final public String toString() {
		return String.format("%-20s\t%-10s\t%-40s\t%-40s\t%15s", tipo(), targa, casaProduttrice, modello, prezzo);
	}
	
	final public String toSaveFormat() {
		return String.format("%s;%s;%s;%s;%s", tipo(), targa, casaProduttrice, modello, prezzo);
	}
	
	final public Targa targa() {
		return targa;
	}
	
	final public CasaProduttrice casaProduttrice() {
		return casaProduttrice;
	}
	
	final public ModelloVeicolo modello() {
		return modello;
	}
	
	final public Prezzo prezzo() {
		return prezzo;
	}
	
	public Prezzo prezzoFinale() {
		return prezzo;
	}
	
	// Improvement by Salvatore Laiso: Let's get a compilation error if we forget to define equals() in subclasses. Let's promote fail-fast approaches.
//	@Override
//	public boolean equals(Object obj) {
//		throw new IllegalStateException("Subclasses of Veicolo must define equals()");
//	}
	public abstract boolean equals(Object obj);
	
	// FIXME: we should also force the definition of hashCode() whenever we redefine equals()
}
